local Me=CreateFrame("Frame","vZifnabsLocator")
vZifnabs.mod.locator=Me
local mod=vZifnabs.mod
local db=nil
local L={}

local widget={pfx="vZifnabs_Locator"}
local zone_name={}
local config={}

local FRACT=1000
local POS_MULTI=100*FRACT
local ZONE_MULTI=POS_MULTI*POS_MULTI

local MMDEFAULTSCALE={X=6.2,Y=4.8}
local ZOOMSCALE={[0]=1,[1]=1.1,[2]=1.3,[3]=1.7,[4]=2.3,[5]=3.3}

-- -----------------------------------------------------------------
-- INITIALIZE
-- -----------------------------------------------------------------
function Me:Initialize()
	db,L,active=mod.core:InitStart("locator")
	if not db.locator then db.locator={mmscale={}} end

	-- load zone names by numbers for quicker reference
	for name,entry in pairs(L.locator.zones) do
		zone_name[entry.nr]=name
	end
	
	-- Check for not permanently defined zones
	if db.locator.unknownzones then
		local udef=0
		for name,entry in pairs(db.locator.unknownzones) do
			if name~="next_nr" then
				zone_name[entry.nr]=name
				local count=0
				for name,info in pairs(db.targets.seen) do
					if info.pos then
						for nr,pos in pairs(info.pos) do
							local x,y,zone,zonenr=Me:ParseLoc(pos)
							if zonenr==entry.nr then count=count+1 end
						end
					end
				end
				if count>0 then 
					mod.core:Print(mod.core:Repl(L.msg.undefzonelist,{"<name>","<nr>","<count>"},{name,entry_nr,count}))
					udef=udef+1
				else
					mod.core:Print(mod.core:Repl(L.msg.undefzonedel,{"<name>","<nr>"},{name,entry_nr}))
					db.locator.unknownzones[name]=nil
				end
			end
		end
		if udef==0 then db.locator.unknownzones=nil end
	end
	
	-- Check for not permanently defined mm scales
	if db.locator.mmscale then
		local lz=L.locator.zones
		for zone,scale in pairs(db.locator.mmscale) do
			local dbscale="(x="..(scale.x or MMDEFAULTSCALE.X)..", y="..(scale.y or MMDEFAULTSCALE.Y)..")"
			local lzscale="(x="..MMDEFAULTSCALE.X..", y="..MMDEFAULTSCALE.Y..")"
			if lz[zone] then lzscale="(x="..(lz[zone].x or MMDEFAULTSCALE.X)..", y="..(lz[zone].y or MMDEFAULTSCALE.Y)..")" end
			if not lz[zone] then 
				mod.core:Print("INTERNAL ERROR: db mmscale overwrite "..dbscale.." for unexistant zone "..zone)
			elseif dbscale==lzscale then
				db.locator.mmscale[zone]=nil
			else
				mod.core:Print(mod.core:Repl(L.msg.scaleovw,{"<dbscale>","<zone>","<lzscale>"},{dbscale,zone,lzscale}))
			end
		end
	end
	
	widget.postext=getglobal(widget.pfx.."Pos_Text")
	widget.postext:SetTextColor(1,1,1,1)
	
	Me:ConfigInitialize()
end
	
function Me:Activate()	
	widget.postext:Show()
	mod.core:RegisterEvent("TICK","locator",Me.Tick)
end

function Me:Deactivate()
	mod.core:DeRegisterEvent("TICK","locator")
	widget.postext:Hide()
end

-- -----------------------------------------------------------------
-- TICK
-- -----------------------------------------------------------------
function Me:Tick()
	local x,y,zone=Me:GetLoc()	
	x=math.floor((100*x+5)/10)/10
	y=math.floor((100*y+5)/10)/10
	if not string.find(" "..x,"[.]") then x=x..".0" end
	if not string.find(" "..y,"[.]") then y=y..".0" end
	widget.postext:SetText(x.." / "..y)	
end

-- -----------------------------------------------------------------
-- GET LOCATION
-- -----------------------------------------------------------------
function Me:GetLoc(zone, x, y)
	if not zone then 
		zone=Me:GetZoneName()
		x, y = GetPlayerMapPosition("player")
	end
	local zone_nr=Me:GetZoneNr(zone)
	x=math.min(POS_MULTI-1,math.floor(POS_MULTI*x+0.5))
	y=math.min(POS_MULTI-1,math.floor(POS_MULTI*y+0.5))
	return x/FRACT,y/FRACT,zone,ZONE_MULTI*(zone_nr or 0)+POS_MULTI*x+y,zone_nr
end

-- -----------------------------------------------------------------
-- PARSE LOCATION NUMBER
-- -----------------------------------------------------------------
function Me:ParseLoc(loc)
	local zone_nr=math.floor(loc/ZONE_MULTI)
	loc=loc-zone_nr*ZONE_MULTI
	local x=math.floor(loc/POS_MULTI)
	local y=loc-x*POS_MULTI
	return x/FRACT,y/FRACT,zone_name[zone_nr],zone_nr
end

-- -----------------------------------------------------------------
-- GET ZONE NUMBER
-- -----------------------------------------------------------------
function Me:GetZoneNr(zone)
	local zone_nr=nil
	if L.locator.zones[zone] then 
		zone_nr=L.locator.zones[zone].nr
	elseif db.locator.unknownzones and db.locator.unknownzones[zone] then
		zone_nr=db.locator.unknownzones[zone].nr
	elseif zone and zone~="" then
		if not db.locator.unknownzones then db.locator.unknownzones={next_nr=999} end
		zone_nr=db.locator.unknownzones.next_nr
		db.locator.unknownzones.next_nr=db.locator.unknownzones.next_nr-1
		db.locator.unknownzones[zone]={nr=zone_nr,addby=UnitName("player"),time=time()}
	end
	return zone_nr
end

-- -----------------------------------------------------------------
-- GET ZONE NUMBER
-- -----------------------------------------------------------------
function Me:GetZoneName(continent,zonenr)
	local here=(not zonenr)
	if here then
		SetMapToCurrentZone()
		zonenr=GetCurrentMapZone()
		continent=GetCurrentMapContinent()
	end
	local zones={GetMapZones(continent)}
	local zone=zones[zonenr]
	if here and not zone then zone=GetZoneText() end
	return zone
end

-- -----------------------------------------------------------------
-- GET MINIMAP SCALE
-- -----------------------------------------------------------------
function Me:GetMinimapScale(zone)
	if not zone then zone=Me:GetZoneName() end
	local scalex=MMDEFAULTSCALE.X
	local scaley=MMDEFAULTSCALE.Y
	if L.locator.zones[zone] then
		scalex=L.locator.zones[zone].x or scalex
		scaley=L.locator.zones[zone].y or scaley
	end
	if db.locator.mmscale and db.locator.mmscale[zone] then
		scalex=db.locator.mmscale[zone].x or scalex
		scaley=db.locator.mmscale[zone].y or scaley
	end
	if config.scalezone==zone then
		scalex=config.val.scalex.now or scalex
		scaley=config.val.scaley.now or scaley
	end
	local zoom=Minimap:GetZoom()
	return scalex,scaley,scalex/ZOOMSCALE[zoom],scaley/ZOOMSCALE[zoom]
end

-- -----------------------------------------------------------------
-- CONFIGURATION INITIALIZE
-- -----------------------------------------------------------------
function Me:ConfigInitialize()
	local pfx=widget.pfx.."_Config"

	widget.mmscalex=mod.config:InitializeSlider(pfx.."_MMScale_X",1,400,1,10,L.locator.mmscale.x.text,L.locator.mmscale.x.tt)
	widget.mmscalex.slider.exitmodule=Me
	widget.mmscalex.slider.valuename="scalex"
	widget.mmscaley=mod.config:InitializeSlider(pfx.."_MMScale_Y",1,400,1,10,L.locator.mmscale.y.text,L.locator.mmscale.y.tt)
	widget.mmscaley.slider.exitmodule=Me
	widget.mmscaley.slider.valuename="scaley"
	getglobal(pfx.."_MMScale_Text"):SetText(L.locator.mmscale.text)
	widget.mmzone=getglobal(pfx.."_MMScale_Zone")
	
	mod.config:Register(Me,L.button.locator.config,"vZifnabs_Locator_Config")
end

-- -----------------------------------------------------------------
-- CONFIGURATION START
-- -----------------------------------------------------------------
function Me:ConfigStart()
	local scalex,scaley=Me:GetMinimapScale(config.scalezone)
	config={prepare=true,scalezone=Me:GetZoneName(),val={
		scalex={init=scalex,now=scalex},
		scaley={init=scaley,now=scaley},
	}}
	widget.mmzone:SetText(config.scalezone)
	widget.mmscalex.slider:SetValue(10*scalex)
	widget.mmscaley.slider:SetValue(10*scaley)
	config.prepare=false
end

-- -----------------------------------------------------------------
-- CONFIGURATION SAVE
-- -----------------------------------------------------------------
function Me:ConfigSave()
	local mmzone=config.scalezone
	db.locator.mmscale[mmzone]={x=config.val.scalex.now,y=config.val.scaley.now}
	mod.core:Print(mod.core:Repl(L.msg.mmscale,{"<zone>","<x>","<y>"},{mmzone,db.locator.mmscale[mmzone].x,db.locator.mmscale[mmzone].y}))
end

-- -----------------------------------------------------------------
-- CONFIGURATION DROP & STOP
-- -----------------------------------------------------------------
function Me:ConfigDrop() Me:ConfigStart() end
function Me:ConfigStop() config={} end

-- -----------------------------------------------------------------
-- SLIDER VALUE CHANGED
-- -----------------------------------------------------------------
function Me:SliderValueChanged(widget,effectivevalue)
	config.val[widget.valuename].now=effectivevalue
	local changes=false
	for name,data in pairs(config.val) do changes=changes or data.init~=data.now end
	mod.config:SetUnsavedChanges(changes) 
end